<?php
// $Id: FeedsYoutubeParser.inc,v 1.2 2010/12/19 09:56:46 doublethink Exp $

/**
 * @file
 *  Feeds parser class for Youtube

/**
 * Class definition for Youtube Parser.
 *
 * Parses RSS and Atom feeds.
 */
class FeedsYoutubeParser extends FeedsParser {
  /**
   * Parse the extra mapping sources provided by this parser.
   */
  public function parse(FeedsImportBatch $batch, FeedsSource $source) {
    $youtube_feed = $batch->getRaw();
    $sxml = simplexml_load_string($youtube_feed);
    $items = array();
    $feed_title = $sxml->title;
    
    // Iterate over entries in feed
    foreach ($sxml->entry as $entry) {
      // get video ID
      $arr = explode('/',$entry->id);
      $id = $arr[count($arr)-1];
      
      // get nodes in media: namespace for media information
      $media = $entry->children('http://search.yahoo.com/mrss/');

      // get video player URL
      $attrs = $media->group->player->attributes();
      $watch = str_replace('&feature=youtube_gdata_player', '', $attrs['url']);
      
      // get video thumbnail
      $attrs = $media->group->thumbnail[0]->attributes();
      $thumbnail = $attrs['url']; 
      
      // get <yt:duration> node for video length
      $yt = $media->children('http://gdata.youtube.com/schemas/2007');
      $attrs = $yt->duration->attributes();
      $length = $attrs['seconds'];

      // get <yt:stats> node for viewer statistics
      $yt = $entry->children('http://gdata.youtube.com/schemas/2007');
      $attrs = $yt->statistics->attributes();
      $viewCount = $attrs['viewCount'];
      $favCount = $attrs['favoriteCount'];  
      
      // get <gd:rating> node for video ratings
      $gd = $entry->children('http://schemas.google.com/g/2005'); 
      if ($gd->rating) {
        $attrs = $gd->rating->attributes();
        $rating = $attrs['average']; 
      } else {
        $rating = 0; 
      } 
      
      $items[] = array(
        'fead_title' => $feed_title,
        'video_id' => $id,
        'title' => $media->group->title,
        'author' => $entry->author->name,
        'description' => $media->group->description,
        'thumbnail' => $thumbnail,
        'category' => $media->group->category,
        'tags' => explode(',', $media->group->keywords),
        'embedded_player' => '',
        'watch_page' => $watch,
        'duration' => $this->secsToTime($length),
        'view_count' => $viewCount,
        'fav_count' => $viewCount,
        'rating' => $rating,
        'updated_datetime' => $entry->updated,
        'published_datetime' => $entry->published
      );
  
      // Populate the FeedsImportBatch object with the parsed results.
      $batch->setTitle($xml->title);
      $batch->setItems($items); 
    }
  }

  /**
   * Add the extra mapping sources provided by this parser.
   */
  public function getMappingSources() {
    return parent::getMappingSources() + array(
      'feed_title' => array(
        'name' => t('Feed title'),
        'description' => t('The title from the pulled feed.'),
      ),
      'video_id' => array(
        'name' => t('Video ID'),
        'description' => t('Video unique ID.'),
      ),
      'title' => array(
        'name' => t('Video title'),
        'description' => t('Video title.'),
      ),
      'author' => array(
        'name' => t('Author'),
        'description' => t('Author or uploader of the video.'),
      ),
      'updated_datetime' => array(
        'name' => t('Updated on (Datetime)'),
      ),
      'published_datetime' => array(
        'name' => t('Published on (Datetime)'),
      ),
      'description' => array(
        'name' => t('Description'),
      ),
      'thumbnail' => array(
        'name' => t('Thumbnail'),
      ),
      'category' => array(
        'name' => t('Category'),
      ),
      'tags' => array(
        'name' => t('Tags'),
      ),
      'watch_page' => array(
        'name' => t('Watch page'),
      ),
      'duration' => array(
        'name' => t('Duration'),
      ),
      'fav_count' => array(
        'name' => t('Favorite count'),
      ),
      'view_count' => array(
        'name' => t('View count'),
      ),
      'rating' => array(
        'name' => t('Rating'),
      )
    );
  }
  
  /**
   *  Display seconds as HH:MM:SS, with leading 0's.
   *
   *  @param $seconds
   *    The number of seconds to display.
   */
  public function secsToTime($seconds) {
    // Number of seconds in an hour.
    $unith =3600;
    // Number of seconds in a minute.
    $unitm =60;
  
    // '/' given value by num sec in hour... output = HOURS
    $hh = intval($seconds / $unith);
  
    // Multiply number of hours by seconds, then subtract from given value.
    // Output = REMAINING seconds.
    $ss_remaining = ($seconds - ($hh * 3600));
  
    // Take remaining seconds and divide by seconds in a min... output = MINS.
    $mm = intval($ss_remaining / $unitm);
    // Multiply number of mins by seconds, then subtract from remaining seconds.
    // Output = REMAINING seconds.
    $ss = ($ss_remaining - ($mm * 60));
  
    $output = '';
    
    // If we have any hours, then prepend that to our output.
    if ($hh) {
      $output .= "$hh:";
    }
  
    // Create a safe-for-output MM:SS.
    $output .= check_plain(sprintf($hh ? "%02d:%02d" : "%d:%02d", $mm, $ss));
  
    return $output;
  }
}